@@ -8,6 +8,7 @@
 #include "kmenuedit.h"
 
 #include <QFrame>
+#include <QMenuBar>
 #include <QSplitter>
 #include <QVBoxLayout>
 
@@ -15,8 +16,9 @@
 #include <KActionMenu>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KStandardAction>
+#include <KStandardActions>
 #include <KStandardShortcut>
+#include <KToolBar>
 #include <KTreeWidgetSearchLine>
 #include <KXMLGUIFactory>
 #include <QAction>
@@ -50,6 +52,67 @@ KMenuEdit::~KMenuEdit()
 
 void KMenuEdit::setupActions()
 {
+    // hamburger & menubar
+    const bool menuBarInitiallyVisible = ConfigurationManager::getInstance()->menuBarVisible();
+    menuBar()->setVisible(menuBarInitiallyVisible);
+
+    auto showMenuBarAction = KStandardAction::showMenubar(this, nullptr, actionCollection());
+    showMenuBarAction->setChecked(menuBarInitiallyVisible);
+    connect(showMenuBarAction, &QAction::toggled, this, [this, &showMenuBarAction]() {
+        const bool newVisibility = !menuBar()->isVisible();
+
+        if (!newVisibility && toolBar()->isHidden()) {
+            const QString accel = showMenuBarAction->shortcut().toString(QKeySequence::NativeText);
+            KMessageBox::information(this,
+                                     i18n("This will hide the menu bar completely."
+                                          " You can show it again by typing %1.",
+                                          accel),
+                                     i18n("Hide menu bar"),
+                                     QStringLiteral("HideMenuBarWarning"));
+        }
+
+        menuBar()->setVisible(newVisibility);
+        ConfigurationManager::getInstance()->setMenuBarVisible(newVisibility);
+    });
+
+    auto hamburgerMenu = KStandardAction::hamburgerMenu(nullptr, nullptr, actionCollection());
+    hamburgerMenu->setMenuBar(menuBar());
+    hamburgerMenu->setMenuBarAdvertised(true);
+    hamburgerMenu->setShowMenuBarAction(showMenuBarAction);
+
+    connect(hamburgerMenu, &KHamburgerMenu::aboutToShowMenu, this, [this]() {
+        auto hamburgerMenu = actionCollection()->action(QStringLiteral("hamburger_menu"));
+
+        auto menu = hamburgerMenu->menu();
+        if (!menu) {
+            menu = new QMenu(this);
+            hamburgerMenu->setMenu(menu);
+        } else {
+            menu->clear();
+        }
+
+        menu->addAction(actionCollection()->action(NEW_ACTION_NAME));
+        menu->addAction(actionCollection()->action(SAVE_ACTION_NAME));
+        menu->addSeparator();
+        menu->addAction(actionCollection()->action(MOVE_UP_ACTION_NAME));
+        menu->addAction(actionCollection()->action(MOVE_DOWN_ACTION_NAME));
+        menu->addAction(actionCollection()->action(CUT_ACTION_NAME));
+        menu->addAction(actionCollection()->action(COPY_ACTION_NAME));
+        menu->addAction(actionCollection()->action(COPY_FILEPATH_ACTION_NAME));
+        menu->addAction(actionCollection()->action(PASTE_ACTION_NAME));
+        menu->addAction(actionCollection()->action(SORT_ACTION_NAME));
+        menu->addAction(actionCollection()->action(OPEN_CONTAINING_FOLDER_ACTION_NAME));
+        menu->addAction(actionCollection()->action(PROPERTIES_ACTION_NAME));
+        menu->addSeparator();
+        menu->addAction(actionCollection()->action(QStringLiteral("restore_system_menu")));
+        menu->addSeparator();
+
+        for (KToolBar *toolbar : toolBars()) {
+            static_cast<KHamburgerMenu *>(hamburgerMenu)->hideActionsOf(toolbar);
+        }
+    });
+
+    // File menu
     QAction *action = actionCollection()->addAction(NEW_SUBMENU_ACTION_NAME);
     action->setIcon(QIcon::fromTheme(QStringLiteral("menu_new")));
     action->setText(i18n("&New Submenu..."));
@@ -63,6 +126,26 @@ void KMenuEdit::setupActions()
     action->setText(i18n("New S&eparator"));
     actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::Key_I));
 
+    // File actions menu
+    action = actionCollection()->addAction(COPY_FILEPATH_ACTION_NAME);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy-path")));
+    action->setText(i18nc("@action:inmenu", "Copy Location"));
+    action = actionCollection()->addAction(OPEN_CONTAINING_FOLDER_ACTION_NAME);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("document-open-folder")));
+    action->setText(i18nc("@action:inmenu", "&Open Containing Folder"));
+    action = actionCollection()->addAction(PROPERTIES_ACTION_NAME);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
+    action->setText(i18nc("@action:inmenu", "Properties"));
+    actionCollection()->setDefaultShortcuts(action, {Qt::ALT | Qt::Key_Return, Qt::ALT | Qt::Key_Enter});
+
+    // "new" menu
+    KActionMenu *newMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("list-add-symbolic")), i18n("&New"), this);
+    newMenu->setPopupMode(QToolButton::InstantPopup);
+    newMenu->addAction(actionCollection()->action(NEW_ITEM_ACTION_NAME));
+    newMenu->addAction(actionCollection()->action(NEW_SUBMENU_ACTION_NAME));
+    newMenu->addAction(actionCollection()->action(NEW_SEPARATOR_ACTION_NAME));
+    actionCollection()->addAction(NEW_ACTION_NAME, newMenu);
+
     // "sort selection" menu
     KActionMenu *sortMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("view-sort-ascending")), i18n("&Sort"), this);
     sortMenu->setPopupMode(QToolButton::InstantPopup);
@@ -89,17 +172,18 @@ void KMenuEdit::setupActions()
     action->setIcon(QIcon::fromTheme(QStringLiteral("go-down")));
     action->setText(i18n("Move &Down"));
 
-    actionCollection()->addAction(KStandardAction::Save, this, SLOT(slotSave()));
-    actionCollection()->addAction(KStandardAction::Quit, this, SLOT(close()));
-    actionCollection()->addAction(KStandardAction::Cut);
-    actionCollection()->addAction(KStandardAction::Copy);
-    actionCollection()->addAction(KStandardAction::Paste);
+    actionCollection()->addAction(KStandardActions::Save, this, &KMenuEdit::slotSave);
+    actionCollection()->addAction(KStandardActions::Quit, this, &KMenuEdit::close);
+    actionCollection()->addAction(KStandardActions::Cut);
+    actionCollection()->addAction(KStandardActions::Copy);
+    actionCollection()->addAction(KStandardActions::Paste);
 
     action = new QAction(i18n("Restore to System Menu"), this);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("edit-reset-symbolic")));
     actionCollection()->addAction(QStringLiteral("restore_system_menu"), action);
     connect(action, &QAction::triggered, this, &KMenuEdit::slotRestoreMenu);
 
-    KStandardAction::preferences(this, SLOT(slotConfigure()), actionCollection());
+    KStandardActions::preferences(this, &KMenuEdit::slotConfigure, actionCollection());
 }
 
 void KMenuEdit::slotConfigure()
